@@ -4,36 +4,45 @@
 #include <glib/gi18n.h>
 #include "gt-twitch.h"
 #include "gt-player.h"
-#include "gt-player-clutter.h"
-//#include "gt-player-mpv.h"
 #include "gt-player-header-bar.h"
 #include "gt-browse-header-bar.h"
 #include "gt-channels-view.h"
 #include "gt-games-view.h"
-#include "gt-favourites-view.h"
+#include "gt-follows-view.h"
 #include "gt-settings-dlg.h"
 #include "gt-twitch-login-dlg.h"
 #include "gt-twitch-channel-info-dlg.h"
 #include "gt-chat.h"
 #include "gt-enums.h"
-#include "utils.h"
 #include "config.h"
 #include "version.h"
+#include "utils.h"
+
+#define TAG "GtWin"
+#include "gnome-twitch/gt-log.h"
 
 #define MAIN_VISIBLE_CHILD gtk_stack_get_visible_child(GTK_STACK(priv->main_stack))
 
+typedef struct
+{
+    gchar* msg;
+    GCallback cb;
+    gpointer udata;
+} QueuedInfoData;
+
 typedef struct
 {
     GtkWidget* main_stack;
     GtkWidget* channels_view;
     GtkWidget* games_view;
-    GtkWidget* favourites_view;
+    GtkWidget* follows_view;
     GtkWidget* header_stack;
     GtkWidget* browse_stack;
     GtkWidget* player_header_bar;
     GtkWidget* browse_header_bar;
     GtkWidget* browse_stack_switcher;
     GtkWidget* chat_view;
+    GtkWidget* player;
 
     GtkWidget* info_revealer;
     GtkWidget* info_label;
@@ -41,16 +50,15 @@ typedef struct
     GtkWidget* info_bar_yes_button;
     GtkWidget* info_bar_no_button;
     GtkWidget* info_bar_ok_button;
-    GCallback info_bar_cb;
-    gpointer info_bar_udata;
-    gboolean info_bar_queued;
+    GtkWidget* info_bar_details_button;
+    GtkWidget* info_bar_close_button;
+
+    QueuedInfoData* cur_info_data;
+    GQueue* info_queue;
 
     GtSettingsDlg* settings_dlg;
 
     gboolean fullscreen;
-
-
-    GtChannel* open_channel;
 } GtWinPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE(GtWin, gt_win, GTK_TYPE_APPLICATION_WINDOW)
@@ -60,8 +68,8 @@ enum
     PROP_0,
     PROP_CHANNELS_VIEW,
     PROP_GAMES_VIEW,
-    PROP_FULLSCREEN,
     PROP_VISIBLE_VIEW,
+    PROP_FULLSCREEN,
     NUM_PROPS
 };
 
@@ -97,9 +105,10 @@ show_about_cb(GSimpleAction* action,
                  "license-type", GTK_LICENSE_GPL_3_0,
                  "copyright", "Copyright © 2015 Vincent Szolnoky",
                  "comments", _("Enjoy Twitch on your GNU/Linux desktop"),
-                 "logo-icon-name", "gnome-twitch",
-                 "website", "https://github.com/Ippytraxx/gnome-twitch",
+                 "logo-icon-name", "com.vinszent.GnomeTwitch",
+                 "website", "https://github.com/vinszent/gnome-twitch",
                  "website-label", "GitHub",
+                 // Translators: Put your details here :)
                  "translator-credits", _("translator-credits"),
                  NULL);
 
@@ -124,6 +133,18 @@ show_settings_cb(GSimpleAction* action,
         g_object_add_weak_pointer(G_OBJECT(priv->settings_dlg), (gpointer *) &priv->settings_dlg);
     }
 
+    if (par)
+    {
+        GEnumClass* eclass = g_type_class_ref(GT_TYPE_SETTINGS_DLG_VIEW);
+        GEnumValue* eval = g_enum_get_value_by_nick(eclass, g_variant_get_string(par, NULL));
+
+        gt_settings_dlg_set_view(priv->settings_dlg, eval->value);
+
+        g_simple_action_set_state(action, par);
+
+        g_type_class_unref(eclass);
+    }
+
     gtk_window_present(GTK_WINDOW(priv->settings_dlg));
 }
 
@@ -143,6 +164,93 @@ refresh_login_cb(GtkInfoBar* info_bar,
     }
 }
 
+static void
+show_error_dialogue_cb(GtkInfoBar* info_bar,
+                       gint res,
+                       gchar** udata)
+{
+    GtkBuilder* builder = gtk_builder_new_from_resource("/com/vinszent/GnomeTwitch/ui/gt-error-dlg.ui");
+    GtkWidget* dlg = GTK_WIDGET(gtk_builder_get_object(builder, "dlg"));
+    GtkTextView* details_text_view = GTK_TEXT_VIEW(gtk_builder_get_object(builder, "details_text_view"));
+
+    if (res != GTK_RESPONSE_OK) goto cleanup;
+
+    gtk_text_buffer_set_text(gtk_text_view_get_buffer(details_text_view),
+                             *(udata+1), -1);
+
+    g_object_set(dlg,
+                 "text", _("Something went wrong"),
+                 "secondary-text", *udata,
+                 NULL);
+    g_signal_connect(dlg, "response", G_CALLBACK(gtk_widget_destroy), NULL);
+
+    gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(GT_WIN_ACTIVE));
+    gtk_widget_show(dlg);
+
+cleanup:
+    g_strfreev(udata);
+}
+
+
+static void
+show_info_bar(GtWin* self)
+{
+    GtWinPrivate* priv = gt_win_get_instance_private(self);
+    QueuedInfoData* data = g_queue_pop_head(priv->info_queue);
+
+    if (data)
+    {
+        if (data->cb)
+        {
+            if (data->cb == G_CALLBACK(show_error_dialogue_cb))
+            {
+                gtk_widget_set_visible(priv->info_bar_ok_button, FALSE);
+                gtk_widget_set_visible(priv->info_bar_yes_button, FALSE);
+                gtk_widget_set_visible(priv->info_bar_no_button, FALSE);
+                gtk_widget_set_visible(priv->info_bar_details_button, TRUE);
+                gtk_widget_set_visible(priv->info_bar_close_button, TRUE);
+                gtk_label_set_markup(GTK_LABEL(priv->info_label), data->msg);
+                gtk_info_bar_set_message_type(GTK_INFO_BAR(priv->info_bar), GTK_MESSAGE_ERROR);
+
+                g_signal_connect(priv->info_bar, "response", G_CALLBACK(data->cb), data->udata);
+            }
+            else
+            {
+
+                g_signal_connect(priv->info_bar, "response", G_CALLBACK(data->cb), data->udata);
+
+                gtk_widget_set_visible(priv->info_bar_ok_button, FALSE);
+                gtk_widget_set_visible(priv->info_bar_yes_button, TRUE);
+                gtk_widget_set_visible(priv->info_bar_no_button, TRUE);
+                gtk_widget_set_visible(priv->info_bar_details_button, FALSE);
+                gtk_widget_set_visible(priv->info_bar_close_button, FALSE);
+                gtk_label_set_markup(GTK_LABEL(priv->info_label), data->msg);
+                gtk_info_bar_set_message_type(GTK_INFO_BAR(priv->info_bar), GTK_MESSAGE_QUESTION);
+            }
+        }
+        else
+        {
+            gtk_widget_set_visible(priv->info_bar_yes_button, FALSE);
+            gtk_widget_set_visible(priv->info_bar_no_button, FALSE);
+            gtk_widget_set_visible(priv->info_bar_ok_button, TRUE);
+            gtk_widget_set_visible(priv->info_bar_details_button, FALSE);
+            gtk_widget_set_visible(priv->info_bar_close_button, FALSE);
+            gtk_label_set_markup(GTK_LABEL(priv->info_label), data->msg);
+            gtk_info_bar_set_message_type(GTK_INFO_BAR(priv->info_bar), GTK_MESSAGE_INFO);
+        }
+
+        priv->cur_info_data = data;
+
+        gtk_revealer_set_reveal_child(GTK_REVEALER(priv->info_revealer), TRUE);
+    }
+    else
+    {
+        priv->cur_info_data = NULL;
+
+        gtk_revealer_set_reveal_child(GTK_REVEALER(priv->info_revealer), FALSE);
+    }
+}
+
 static void
 close_info_bar_cb(GtkInfoBar* bar,
                   gint res,
@@ -151,18 +259,20 @@ close_info_bar_cb(GtkInfoBar* bar,
     GtWin* self = GT_WIN(udata);
     GtWinPrivate* priv = gt_win_get_instance_private(self);
 
-    if (!priv->info_bar_queued)
-        gtk_revealer_set_reveal_child(GTK_REVEALER(priv->info_revealer), FALSE);
-    priv->info_bar_queued = FALSE;
-
-    if (priv->info_bar_cb)
+    if (priv->cur_info_data)
     {
-        g_signal_handlers_disconnect_by_func(priv->info_bar,
-                                             priv->info_bar_cb,
-                                             priv->info_bar_udata);
-        priv->info_bar_cb = NULL;
-        priv->info_bar_udata = NULL;
+        if (priv->cur_info_data->cb)
+        {
+            g_signal_handlers_disconnect_by_func(priv->info_bar,
+                                                 priv->cur_info_data->cb,
+                                                 priv->cur_info_data->udata);
+        }
+
+        g_free(priv->cur_info_data->msg);
+        g_free(priv->cur_info_data);
     }
+
+    show_info_bar(self);
 }
 
 
@@ -203,15 +313,20 @@ show_channel_info_cb(GSimpleAction* action,
     GtWin* self = GT_WIN(udata);
     GtWinPrivate* priv = gt_win_get_instance_private(self);
     GtTwitchChannelInfoDlg* dlg = gt_twitch_channel_info_dlg_new(GTK_WINDOW(self));
-    const gchar* chan;
+    GtChannel* channel;
+    const gchar* name;
 
-    chan = gt_channel_get_name(priv->open_channel);
+    g_object_get(self->player, "channel", &channel, NULL);
 
-    g_message("{GtWin} Showing channel info for '%s'", chan);
+    name = gt_channel_get_name(channel);
+
+    g_message("{GtWin} Showing channel info for '%s'", name);
 
     gtk_window_present(GTK_WINDOW(dlg));
 
-    gt_twitch_channel_info_dlg_load_channel(dlg, chan);
+    gt_twitch_channel_info_dlg_load_channel(dlg, name);
+
+    g_object_unref(channel);
 }
 
 
@@ -222,11 +337,20 @@ refresh_view_cb(GSimpleAction* action,
 {
     GtWin* self = GT_WIN(udata);
     GtWinPrivate* priv = gt_win_get_instance_private(self);
+    GtkWidget* visible_child = gtk_stack_get_visible_child(GTK_STACK(priv->browse_stack));
 
-    if (gtk_stack_get_visible_child(GTK_STACK(priv->browse_stack)) == priv->channels_view)
+    if (visible_child == priv->channels_view)
         gt_channels_view_refresh(GT_CHANNELS_VIEW(priv->channels_view));
-    else if (gtk_stack_get_visible_child(GTK_STACK(priv->browse_stack)) == priv->games_view)
+    else if (visible_child == priv->games_view)
         gt_games_view_refresh(GT_GAMES_VIEW(priv->games_view));
+    else if (visible_child == priv->follows_view)
+    {
+        //TODO: Quick hack, turn this into a proper refresh function
+        if (gt_app_credentials_valid(main_app))
+            gt_follows_manager_load_from_twitch(main_app->fav_mgr);
+        else
+            gt_follows_manager_load_from_file(main_app->fav_mgr);
+    }
 }
 
 static void
@@ -256,12 +380,12 @@ key_press_cb(GtkWidget* widget,
 
     g_object_get(self->player, "playing", &playing, NULL);
 
-    if (MAIN_VISIBLE_CHILD == self->player)
+    if (MAIN_VISIBLE_CHILD == GTK_WIDGET(self->player))
     {
         if (evt->keyval == GDK_KEY_Escape)
         {
             if (priv->fullscreen)
-                gtk_window_unfullscreen(GTK_WINDOW(self));
+                g_object_set(self, "fullscreen", FALSE, NULL);
             else
             {
                 action = g_action_map_lookup_action(G_ACTION_MAP(self), "close_player");
@@ -270,10 +394,7 @@ key_press_cb(GtkWidget* widget,
         }
         else if (evt->keyval == GDK_KEY_f)
         {
-            if (priv->fullscreen)
-                gtk_window_unfullscreen(GTK_WINDOW(self));
-            else
-                gtk_window_fullscreen(GTK_WINDOW(self));
+            g_object_set(self, "fullscreen", !priv->fullscreen, NULL);
         }
     }
     else
@@ -290,8 +411,8 @@ key_press_cb(GtkWidget* widget,
                 gt_channels_view_handle_event(GT_CHANNELS_VIEW(priv->channels_view), (GdkEvent*) evt);
             else if (view == priv->games_view)
                 gt_games_view_handle_event(GT_GAMES_VIEW(priv->games_view), (GdkEvent*) evt);
-            else if (view == priv->favourites_view)
-                gt_favourites_view_handle_event(GT_FAVOURITES_VIEW(priv->favourites_view), (GdkEvent* )evt);
+            else if (view == priv->follows_view)
+                gt_follows_view_handle_event(GT_FOLLOWS_VIEW(priv->follows_view), (GdkEvent* )evt);
         }
     }
 
@@ -332,37 +453,29 @@ close_player_cb(GSimpleAction* action,
                                      "browse");
 }
 
+static void
+update_fullscreen(GtWin* self)
+{
+    GtWinPrivate* priv = gt_win_get_instance_private(self);
+
+    if (priv->fullscreen)
+        gtk_window_fullscreen(GTK_WINDOW(self));
+    else
+        gtk_window_unfullscreen(GTK_WINDOW(self));
+}
+
 static GActionEntry win_actions[] =
 {
     {"refresh_view", refresh_view_cb, NULL, NULL, NULL},
     {"show_view_default", show_view_default_cb, NULL, NULL, NULL},
     {"show_about", show_about_cb, NULL, NULL, NULL},
-    {"show_settings", show_settings_cb, NULL, NULL, NULL},
+    {"show_settings", show_settings_cb, NULL, NULL},
+    {"show_settings_with_view", NULL, "s", "'general'", show_settings_cb},
     {"show_twitch_login", show_twitch_login_cb, NULL, NULL, NULL},
     {"show_channel_info", show_channel_info_cb, NULL, NULL, NULL},
     {"close_player", close_player_cb, NULL, NULL, NULL},
 };
 
-static gboolean
-window_state_cb(GtkWidget* widget,
-                GdkEventWindowState* evt,
-                gpointer udata)
-{
-    GtWin* self = GT_WIN(udata);
-    GtWinPrivate* priv = gt_win_get_instance_private(self);
-
-    if (evt->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)
-    {
-        if (evt->new_window_state & GDK_WINDOW_STATE_FULLSCREEN)
-            priv->fullscreen = TRUE;
-        else
-            priv->fullscreen = FALSE;
-        g_object_notify_by_pspec(G_OBJECT(self), props[PROP_FULLSCREEN]);
-    }
-
-    return FALSE;
-}
-
 static void
 finalize(GObject* object)
 {
@@ -370,6 +483,8 @@ finalize(GObject* object)
     GtWinPrivate* priv = gt_win_get_instance_private(self);
 
     G_OBJECT_CLASS(gt_win_parent_class)->finalize(object);
+
+    g_message("{GtWin} Finalise");
 }
 
 static void
@@ -414,6 +529,10 @@ set_property(GObject*      obj,
         case PROP_VISIBLE_VIEW:
             // Do nothing
             break;
+        case PROP_FULLSCREEN:
+            priv->fullscreen = g_value_get_boolean(val);
+            update_fullscreen(self);
+            break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, prop, pspec);
     }
@@ -438,23 +557,23 @@ gt_win_class_init(GtWinClass* klass)
                                                  "Games View",
                                                  GT_TYPE_GAMES_VIEW,
                                                  G_PARAM_READABLE);
-    props[PROP_FULLSCREEN] = g_param_spec_boolean("fullscreen",
-                                                  "Fullscreen",
-                                                  "Whether window is fullscreen",
-                                                  FALSE,
-                                                  G_PARAM_READABLE);
     props[PROP_VISIBLE_VIEW] = g_param_spec_object("visible-view",
                                                    "Visible View",
                                                    "Visible View",
                                                    GTK_TYPE_WIDGET,
                                                    G_PARAM_READWRITE);
+    props[PROP_FULLSCREEN] = g_param_spec_boolean("fullscreen",
+                                                  "Fullscreen",
+                                                  "Whether window is fullscreen",
+                                                  FALSE,
+                                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
 
     g_object_class_install_properties(object_class,
                                       NUM_PROPS,
                                       props);
 
     gtk_widget_class_set_template_from_resource(GTK_WIDGET_CLASS(klass),
-                                                "/com/gnome-twitch/ui/gt-win.ui");
+                                                "/com/vinszent/GnomeTwitch/ui/gt-win.ui");
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, main_stack);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, channels_view);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, games_view);
@@ -464,41 +583,44 @@ gt_win_class_init(GtWinClass* klass)
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, browse_header_bar);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, browse_stack);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, browse_stack_switcher);
-    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, favourites_view);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, follows_view);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, info_revealer);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, info_label);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, info_bar);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, info_bar_yes_button);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, info_bar_no_button);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, info_bar_ok_button);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, info_bar_details_button);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtWin, info_bar_close_button);
 }
 
 static void
 gt_win_init(GtWin* self)
 {
     GtWinPrivate* priv = gt_win_get_instance_private(self);
+    GPropertyAction* action;
 
     GT_TYPE_PLAYER; // Hack to load GtPlayer into the symbols table
-    GT_TYPE_PLAYER_CLUTTER;
-//    GT_TYPE_PLAYER_MPV;
     GT_TYPE_PLAYER_HEADER_BAR;
     GT_TYPE_BROWSE_HEADER_BAR;
     GT_TYPE_CHANNELS_VIEW;
     GT_TYPE_GAMES_VIEW;
-    GT_TYPE_FAVOURITES_VIEW;
+    GT_TYPE_FOLLOWS_VIEW;
     GT_TYPE_CHAT;
 
-//    self->open_channel = NULL;
+    gtk_window_set_application(GTK_WINDOW(self), GTK_APPLICATION(main_app));
+
+    gtk_widget_init_template(GTK_WIDGET(self));
+    gtk_widget_realize(priv->player_header_bar);
+
+    priv->cur_info_data = NULL;
+    priv->info_queue = g_queue_new();
 
     gtk_window_set_default_size(GTK_WINDOW(self),
                                 g_settings_get_int(main_app->settings, "window-width"),
                                 g_settings_get_int(main_app->settings, "window-height"));
 
-    gtk_widget_init_template(GTK_WIDGET(self));
-
-    gtk_window_set_default_icon_name("gnome-twitch");
-
-//    g_object_set(self, "application", main_app, NULL); // Another hack because GTK is bugged and resets the app menu when using custom widgets
+    gtk_window_set_default_icon_name("com.vinszent.GnomeTwitch");
 
     g_object_bind_property(priv->browse_stack, "visible-child",
                            self, "visible-view",
@@ -506,10 +628,10 @@ gt_win_init(GtWin* self)
 
     GdkScreen* screen = gdk_screen_get_default();
     GtkCssProvider* css = gtk_css_provider_new();
-    gtk_css_provider_load_from_resource(css, "/com/gnome-twitch/gnome-twitch-style.css");
-    gtk_style_context_add_provider_for_screen(screen, GTK_STYLE_PROVIDER(css), GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+    gtk_css_provider_load_from_resource(css, "/com/vinszent/GnomeTwitch/com.vinszent.GnomeTwitch.style.css");
+    gtk_style_context_add_provider_for_screen(screen, GTK_STYLE_PROVIDER(css),
+                                              GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
-    g_signal_connect(self, "window-state-event", G_CALLBACK(window_state_cb), self);
     g_signal_connect_after(self, "key-press-event", G_CALLBACK(key_press_cb), self);
     g_signal_connect(self, "delete-event", G_CALLBACK(delete_cb), self);
     g_signal_connect_after(priv->info_bar, "response", G_CALLBACK(close_info_bar_cb), self);
@@ -519,6 +641,11 @@ gt_win_init(GtWin* self)
                                     G_N_ELEMENTS(win_actions),
                                     self);
 
+    action = g_property_action_new("toggle_fullscreen", self, "fullscreen");
+    g_action_map_add_action(G_ACTION_MAP(self),
+                            G_ACTION(action));
+    g_object_unref(action);
+
     GtkWindowGroup* window_group = gtk_window_group_new();
     gtk_window_group_add_window(window_group, GTK_WINDOW(self));
     g_object_unref(window_group);
@@ -530,10 +657,6 @@ gt_win_open_channel(GtWin* self, GtChannel* chan)
 {
     GtWinPrivate* priv = gt_win_get_instance_private(self);
 
-    g_clear_object(&priv->open_channel);
-    g_object_ref(chan);
-    priv->open_channel = chan;
-
     gt_player_open_channel(GT_PLAYER(self->player), chan);
 
     gtk_stack_set_visible_child_name(GTK_STACK(priv->main_stack),
@@ -593,7 +716,7 @@ gt_win_get_games_view(GtWin* self)
 }
 
 gboolean
-gt_win_get_fullscreen(GtWin* self)
+gt_win_is_fullscreen(GtWin* self)
 {
     GtWinPrivate* priv = gt_win_get_instance_private(self);
 
@@ -601,35 +724,55 @@ gt_win_get_fullscreen(GtWin* self)
 }
 
 void
-gt_win_show_info_message(GtWin* self, const gchar* msg)
+gt_win_toggle_fullscreen(GtWin* self)
 {
     GtWinPrivate* priv = gt_win_get_instance_private(self);
 
-    priv->info_bar_queued = gtk_revealer_get_child_revealed(GTK_REVEALER(priv->info_revealer));
+    g_object_set(self, "fullscreen", !priv->fullscreen, NULL);
+}
 
-    gtk_widget_set_visible(priv->info_bar_yes_button, FALSE);
-    gtk_widget_set_visible(priv->info_bar_no_button, FALSE);
-    gtk_widget_set_visible(priv->info_bar_ok_button, TRUE);
-    gtk_label_set_markup(GTK_LABEL(priv->info_label), msg);
-    gtk_info_bar_set_message_type(GTK_INFO_BAR(priv->info_bar), GTK_MESSAGE_INFO);
-    gtk_revealer_set_reveal_child(GTK_REVEALER(priv->info_revealer), TRUE);
+void
+gt_win_show_info_message(GtWin* self, const gchar* msg)
+{
+    GtWinPrivate* priv = gt_win_get_instance_private(self);
+    QueuedInfoData* data = g_new0(QueuedInfoData, 1);
+
+    data->msg = g_strdup(msg);
+
+    g_queue_push_tail(priv->info_queue, data);
+
+    if (!gtk_revealer_get_reveal_child(GTK_REVEALER(priv->info_revealer)))
+        show_info_bar(self);
+}
+
+void
+gt_win_show_error_message(GtWin* self, const gchar* secondary, const gchar* details)
+{
+    GtWinPrivate* priv = gt_win_get_instance_private(self);
+    QueuedInfoData* data = g_new(QueuedInfoData, 1);
+    gchar** udata = g_malloc(sizeof(gchar*)*3);
+    // Translators: Please keep the markup tags
+    gchar* msg = g_strdup_printf(_("<b>Something went wrong:</b> %s."), secondary);
+
+    *udata = g_strdup(secondary);
+    *(udata+1) = g_strdup(details);
+    *(udata+2) = NULL;
+
+    gt_win_ask_question(self, msg, G_CALLBACK(show_error_dialogue_cb), udata);
 }
 
 void
 gt_win_ask_question(GtWin* self, const gchar* msg, GCallback cb, gpointer udata)
 {
     GtWinPrivate* priv = gt_win_get_instance_private(self);
+    QueuedInfoData* data = g_new(QueuedInfoData, 1);
 
-    priv->info_bar_queued = gtk_revealer_get_child_revealed(GTK_REVEALER(priv->info_revealer));
-    priv->info_bar_cb = cb;
-    priv->info_bar_udata = udata;
+    data->msg = g_strdup(msg);
+    data->cb = cb;
+    data->udata = udata;
 
-    g_signal_connect(priv->info_bar, "response", G_CALLBACK(cb), udata);
+    g_queue_push_tail(priv->info_queue, data);
 
-    gtk_widget_set_visible(priv->info_bar_ok_button, FALSE);
-    gtk_widget_set_visible(priv->info_bar_yes_button, TRUE);
-    gtk_widget_set_visible(priv->info_bar_no_button, TRUE);
-    gtk_label_set_text(GTK_LABEL(priv->info_label), msg);
-    gtk_info_bar_set_message_type(GTK_INFO_BAR(priv->info_bar), GTK_MESSAGE_QUESTION);
-    gtk_revealer_set_reveal_child(GTK_REVEALER(priv->info_revealer), TRUE);
+    if (!gtk_revealer_get_reveal_child(GTK_REVEALER(priv->info_revealer)))
+        show_info_bar(self);
 }
